%global kdbversion 8.0

%global major_version %%(echo %{version} | cut -d. -f1)
%global minor_version %%(echo %{version} | cut -d. -f2)

Summary:       Kerberos network authentication protocol
Name:          krb5
Version:       1.21.2
Release:       3%{?dist}
License:       MIT
URL:           https://web.mit.edu/kerberos/www/
Source0:       https://kerberos.org/dist/%{name}/%{major_version}.%{minor_version}/%{name}-%{version}.tar.gz
Source1:       kprop.service
Source2:       kadmin.service
Source3:       krb5kdc.service
Source4:       krb5.conf
Source5:       kdc.conf
Source6:       kadm5.acl
Source7:       krb5kdc.sysconfig
Source8:       kadmin.sysconfig
Source9:       kprop.sysconfig
Source10:      ksu.pamd
Source11:      krb5kdc.logrotate
Source12:      kadmind.logrotate
Source13:      krb5-krb5kdc.conf
Patch3000:     downstream-ksu-pam-integration.patch
Patch3001:     downstream-SELinux-integration.patch
Patch3002:     downstream-netlib-and-dns.patch
Patch3003:     downstream-fix-debuginfo-with-y.tab.c.patch
Patch3004:     Fix-CVE-2024-26458-and-26461-two-unlikely-memory-leaks.patch
Patch3005:     Fix-CVE-2024-26462-leak-in-KDC-NDR-encoding.patch

BuildRequires: autoconf bison make gcc pkgconfig
BuildRequires: gawk sed gzip gettext
BuildRequires: libcom_err-devel libedit-devel libss-devel ncurses-devel
BuildRequires: openssl-devel => 3.0.0
BuildRequires: libselinux-devel pam-devel libverto-devel openldap-devel lmdb-devel
BuildRequires: python3 python3-sphinx perl-interpreter git
BuildRequires: keyutils, keyutils-libs-devel >= 1.5.8
BuildRequires: systemd-units
# for check tests, check failed now
# BuildRequires: dejagnu net-tools rpcbind hostname iproute procps-ng python3-pyrad

%description
Kerberos is a network authentication protocol. It is designed to provide strong authentication
for client/server applications by using secret-key cryptography.The Kerberos protocol uses
strong cryptography so that a client can prove its identity to a server (and vice versa) across
an insecure network connection. After a client and server has used Kerberos to prove their identity,
they can also encrypt all of their communications to assure privacy and data integrity
as they go about their business.


%package libs
Summary:       Shared libraries for Kerberos V5
Requires:      openssl-libs >= 3.0.0
Requires:      keyutils-libs >= 1.5.8
Requires:      /etc/crypto-policies/back-ends/krb5.config

%description libs
Kerberos is a network authentication protocol. The %{name}-libs package
contains the shared libraries needed by Kerberos V5.


%package pkinit
Summary:       PKINIT module for Kerberos V5
Requires:      %{name}-libs = %{version}-%{release}
Provides:      krb5-pkinit-openssl = %{version}-%{release}

%description pkinit
Kerberos is a network authentication protocol. The %{name}-pkinit
package contains the PKINIT plugin, which allows clients to obtain
initial credentials from a KDC using a private key and a certificate.


%package -n libkadm5
Summary:       Administrative libraries for Kerberos V5
Requires:      %{name}-libs = %{version}-%{release}

%description -n libkadm5
Kerberos is a network authentication protocol. The libkadm5 package
contains the libkadm5clnt and libkadm5serv shared objects. 


%package server
Summary:       Server program for Kerberos V5 protocol
BuildRequires: libverto-module-base
Requires:      %{name}-libs = %{version}-%{release}
Requires:      %{name}-pkinit = %{version}-%{release}
Requires:      libkadm5 = %{version}-%{release}
Requires:      libverto-module-base
Requires:      logrotate /usr/share/dict/words
Requires(post): systemd-units
Requires(preun): systemd-units
Requires(postun): systemd-units
Provides:      krb5-kdb-version = %{kdbversion}

%description server
Kerberos is a network authentication protocol. The %{name}-server package
contains KDC(key distribution center),  kadmind and more.


%package server-ldap
Summary:       LDAP storage plugin for Kerberos V5 KDC
Requires:      %{name}-server = %{version}-%{release}

%description server-ldap
Kerberos is a network authentication protocol. The %{name}-server package
provides directory server to store the data for KDC(key distribution center).


%package       client
Summary:       Client programs fro Kerberos V5 protocol
Requires:      %{name}-libs = %{version}-%{release}
Requires:      %{name}-pkinit = %{version}-%{release}
Requires:      libkadm5 = %{version}-%{release}

%description client
Kerberos is a network authentication system. The krb5-workstation
package contains the basic Kerberos programs (kinit, klist, kdestroy,
kpasswd). If your network uses Kerberos, this package should be
installed on every workstation.


%package devel
Summary:       Development files for programs using Kerberos V5
Requires:      %{name}-libs = %{version}-%{release}
Requires:      libkadm5 = %{version}-%{release}
Requires:      libcom_err-devel libverto-devel keyutils-libs-devel libselinux-devel
Provides:      krb5-kdb-devel-version = %{kdbversion}

%description devel
Kerberos is a network authentication protocol. The %{name}-devel package
contains header files and libraries for compiling Kerberos V5 programs.


%prep
%autosetup -S git_am -n %{name}-%{version}


%build
pushd src
autoreconf -fiv
export DEFCCNAME=DIR:/run/user/%%{uid}/krb5cc
%configure \
    CFLAGS="$RPM_OPT_FLAGS $DEFINES -I%{_includedir}/et -fPIC -fno-strict-aliasing -fstack-protector-all" \
    CPPFLAGS="$DEFINES $INCLUDES" \
    SS_LIB="-lss" \
    --enable-shared \
    --runstatedir=/run \
    --localstatedir=%{_var}/kerberos \
    --disable-rpath \
    --without-krb5-config \
    --with-system-et \
    --with-system-ss \
    --with-netlib=-lresolv \
    --enable-dns-for-realm \
    --with-ldap \
    --enable-pkinit \
    --with-crypto-impl=openssl \
    --with-tls-impl=openssl \
    --with-system-verto \
    --with-pam \
    --with-selinux \
    --with-lmdb

%make_build
popd

# Build the docs.
make -C src/doc paths.py version.py
cp src/doc/paths.py doc/
mkdir -p build-man
sphinx-build -a -b man   -t pathsubs doc build-man

%check
pushd src
keyctl session - make check OFFLINE=yes TMPDIR=%{_tmppath} || :
popd

%install
%make_install -C src
sed -r -i -e 's|^libdir=/usr/lib(64)?$|libdir=/usr/lib|g' $RPM_BUILD_ROOT%{_bindir}/krb5-config
sed -i -r -e 's/^(LDFLAGS=).*/\1/' $RPM_BUILD_ROOT%{_bindir}/krb5-config
rm -rf %{buildroot}%{_datadir}/examples
rm -f %{buildroot}/%{_libdir}/krb5/plugins/preauth/test.so

install -pd $RPM_BUILD_ROOT%{_var}/kerberos/krb5/user

install -pd $RPM_BUILD_ROOT%{_unitdir}
for unit in %{SOURCE1} %{SOURCE2} %{SOURCE3}; do
    install -pm 644 ${unit} $RPM_BUILD_ROOT%{_unitdir}
done

install -D -pm 644 %{SOURCE4} $RPM_BUILD_ROOT/etc/krb5.conf
install -pd $RPM_BUILD_ROOT/etc/krb5.conf.d
ln -sv /etc/crypto-policies/back-ends/krb5.config $RPM_BUILD_ROOT/etc/krb5.conf.d/crypto-policies
install -pdm 755 $RPM_BUILD_ROOT/etc/gss
install -pdm 755 $RPM_BUILD_ROOT/etc/gss/mech.d

install -pd $RPM_BUILD_ROOT%{_var}/kerberos/krb5kdc
install -pm 600 %{SOURCE5} %{SOURCE6} $RPM_BUILD_ROOT%{_var}/kerberos/krb5kdc/

for sysconfig in %{SOURCE7} %{SOURCE8} %{SOURCE9}; do
    install -D -pm 644 ${sysconfig} $RPM_BUILD_ROOT/etc/sysconfig/`basename ${sysconfig} .sysconfig`
done

for pam in %{SOURCE10} ; do
    install -D -pm 644 ${pam} $RPM_BUILD_ROOT/etc/pam.d/`basename ${pam} .pamd`
done

for logrotate in %{SOURCE11} %{SOURCE12}; do
    install -D -pm 644 ${logrotate} $RPM_BUILD_ROOT/etc/logrotate.d/`basename ${logrotate} .logrotate`
done

install -pd $RPM_BUILD_ROOT/%{_tmpfilesdir}
install -pm 644 %{SOURCE13} $RPM_BUILD_ROOT/%{_tmpfilesdir}/
install -pd $RPM_BUILD_ROOT/%{_localstatedir}/run/krb5kdc
# Plug-in directories
install -pdm 755 $RPM_BUILD_ROOT/%{_libdir}/krb5/plugins/preauth
install -pdm 755 $RPM_BUILD_ROOT/%{_libdir}/krb5/plugins/kdb
install -pdm 755 $RPM_BUILD_ROOT/%{_libdir}/krb5/plugins/authdata

for section in 1 5 8 ; do
    install -m 644 build-man/*.${section} $RPM_BUILD_ROOT/%{_mandir}/man${section}/
done

%find_lang mit-krb5

%post server
%systemd_post krb5kdc.service kadmin.service kprop.service
/bin/systemctl daemon-reload
exit 0

%preun server
%systemd_preun krb5kdc.service kadmin.service kprop.service
exit 0

%postun server
%systemd_postun_with_restart krb5kdc.service kadmin.service kprop.service
exit 0


%files libs -f mit-krb5.lang
%license NOTICE
%doc README
%dir /etc/gss
%dir /etc/gss/mech.d
%dir /etc/krb5.conf.d
%config(noreplace) /etc/krb5.conf
%config(noreplace,missingok) /etc/krb5.conf.d/crypto-policies
%{_libdir}/libgssapi_krb5.so.*
%{_libdir}/libgssrpc.so.*
%{_libdir}/libk5crypto.so.*
%{_libdir}/libkdb5.so.*
%{_libdir}/libkrad.so.*
%{_libdir}/libkrb5.so.*
%{_libdir}/libkrb5support.so.*
%dir %{_libdir}/krb5
%dir %{_libdir}/krb5/plugins
%dir %{_libdir}/krb5/plugins/*
%{_libdir}/krb5/plugins/tls/k5tls.so
%{_libdir}/krb5/plugins/preauth/spake.so
%dir %{_var}/kerberos
%dir %{_var}/kerberos/krb5
%dir %{_var}/kerberos/krb5/user
%{_mandir}/man5/.k5identity.5*
%{_mandir}/man5/.k5login.5*
%{_mandir}/man5/k5identity.5*
%{_mandir}/man5/k5login.5*
%{_mandir}/man5/krb5.conf.5*
%{_mandir}/man7/kerberos.7*
%docdir %{_mandir}


%files pkinit
%dir %{_libdir}/krb5
%dir %{_libdir}/krb5/plugins
%dir %{_libdir}/krb5/plugins/preauth
%{_libdir}/krb5/plugins/preauth/pkinit.so


%files -n libkadm5
%{_libdir}/libkadm5clnt.so
%{_libdir}/libkadm5clnt_mit.so
%{_libdir}/libkadm5srv.so
%{_libdir}/libkadm5srv_mit.so
%{_libdir}/libkadm5clnt_mit.so.*
%{_libdir}/libkadm5srv_mit.so.*


%files server
%config(noreplace) /etc/sysconfig/krb5kdc
%config(noreplace) /etc/sysconfig/kadmin
%config(noreplace) /etc/sysconfig/kprop
%config(noreplace) /etc/logrotate.d/krb5kdc
%config(noreplace) /etc/logrotate.d/kadmind
%{_unitdir}/krb5kdc.service
%{_unitdir}/kadmin.service
%{_unitdir}/kprop.service
%{_tmpfilesdir}/krb5-krb5kdc.conf
%dir %{_localstatedir}/run/krb5kdc
%docdir %{_mandir}
%doc src/config-files/kdc.conf

%dir %{_var}/kerberos
%dir %{_var}/kerberos/krb5kdc
%config(noreplace) %{_var}/kerberos/krb5kdc/kdc.conf
%config(noreplace) %{_var}/kerberos/krb5kdc/kadm5.acl

%dir %{_libdir}/krb5
%dir %{_libdir}/krb5/plugins
%dir %{_libdir}/krb5/plugins/kdb
%dir %{_libdir}/krb5/plugins/preauth
%dir %{_libdir}/krb5/plugins/authdata
%{_libdir}/krb5/plugins/preauth/otp.so
%{_libdir}/krb5/plugins/kdb/db2.so
%{_libdir}/krb5/plugins/kdb/klmdb.so

%{_sbindir}/kadmin.local
%{_sbindir}/kadmind
%{_sbindir}/kdb5_util
%{_sbindir}/kprop
%{_sbindir}/kpropd
%{_sbindir}/kproplog
%{_sbindir}/krb5kdc
%{_sbindir}/sserver
%{_sbindir}/gss-server
%{_sbindir}/krb5-send-pr
%{_sbindir}/sim_server
%{_sbindir}/uuserver
%{_mandir}/man8/kadmin.local.8*
%{_mandir}/man8/kadmind.8*
%{_mandir}/man8/kdb5_util.8*
%{_mandir}/man8/kprop.8*
%{_mandir}/man8/kpropd.8*
%{_mandir}/man8/kproplog.8*
%{_mandir}/man8/krb5kdc.8*
%{_mandir}/man8/sserver.8*

%{_mandir}/man5/kadm5.acl.5*
%{_mandir}/man5/kdc.conf.5*


%files server-ldap
%dir %{_libdir}/krb5
%dir %{_libdir}/krb5/plugins
%dir %{_libdir}/krb5/plugins/kdb
%{_libdir}/krb5/plugins/kdb/kldap.so
%{_libdir}/libkdb_ldap.so
%{_libdir}/libkdb_ldap.so.*
%{_sbindir}/kdb5_ldap_util
%{_mandir}/man8/kdb5_ldap_util.8.gz
%docdir %{_mandir}
%doc src/plugins/kdb/ldap/libkdb_ldap/kerberos.ldif
%doc src/plugins/kdb/ldap/libkdb_ldap/kerberos.schema


%files client
%config(noreplace) /etc/pam.d/ksu
%{_bindir}/kdestroy
%{_bindir}/kinit
%{_bindir}/klist
%{_bindir}/kpasswd
%{_bindir}/kswitch
%{_bindir}/kvno
%{_bindir}/kadmin
%{_bindir}/k5srvutil
%{_bindir}/ktutil
%{_bindir}/sclient
%{_bindir}/gss-client
%{_bindir}/sim_client
%{_bindir}/uuclient
%attr(4755,root,root) %{_bindir}/ksu
%{_mandir}/man1/kdestroy.1*
%{_mandir}/man1/kinit.1*
%{_mandir}/man1/klist.1*
%{_mandir}/man1/kpasswd.1*
%{_mandir}/man1/kswitch.1*
%{_mandir}/man1/kvno.1*
%{_mandir}/man1/kadmin.1*
%{_mandir}/man1/k5srvutil.1*
%{_mandir}/man1/ktutil.1*
%{_mandir}/man1/sclient.1*
%{_mandir}/man1/ksu.1*
%doc src/config-files/services.append
%doc src/config-files/krb5.conf
%attr(0755,root,root) %doc src/config-files/convert-config-files


%files devel
%{_includedir}/*
%{_libdir}/libgssapi_krb5.so
%{_libdir}/libgssrpc.so
%{_libdir}/libk5crypto.so
%{_libdir}/libkdb5.so
%{_libdir}/libkrad.so
%{_libdir}/libkrb5.so
%{_libdir}/libkrb5support.so
%{_libdir}/pkgconfig/*
%{_bindir}/krb5-config
%{_mandir}/man1/krb5-config.1*
%docdir %{_mandir}


%changelog
* Thu Apr 25 2024 Tao Wu <tallwu@tencent.com> - 1.21.2-3
- [Type] security
- [DESC] fix CVE-2024-26458, CVE-2024-26461, and CVE-2024-26462

* Tue Mar 12 2024 kianli <kianli@tencent.com> - 1.21.2-2
- Remove unnecessary requires for krb5-libs

* Thu Jan 25 2024 Tao Wu <tallwu@tencent.com> - 1.21.2-1
- Update version to 1.21.2

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.20.2-2
- Rebuilt for OpenCloudOS Stream 23.09

* Thu Jul 20 2023 Miaojun Dong <zoedong@tencent.com> - 1.20.2-1
- Bump version to 1.20.2

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.20-5
- Rebuilt for OpenCloudOS Stream 23.05

* Thu Apr 27 2023 Miaojun Dong <zoedong@tencent.com> - 1.20-4
- provide krb5-kdb-version and krb5-kdb-devel-version

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.20-3
- Rebuilt for OpenCloudOS Stream 23

* Wed Dec 28 2022 Shuo Wang <abushwang@tencent.com> - 1.20-2
- fix coredump with unrecognized option

* Fri Jun 10 2022 rockerzhu <rockerzhu@tencent.com> - 1.20-1
- Initial build
